<?php
if (! defined( 'ABSPATH' )) exit;
class ast_addon
{
	public function __construct() {
		add_filter('sAlertDefaultSettings',  __CLASS__ .'::addDefaultSetting',1);
		add_action( 'sa_addTabs', array( $this, 'addTabs' ), 100 );

		$smsalert_ast_notify  = smsalert_get_option( 'ast_notify', 'smsalert_ast_general', 'on');

		if($smsalert_ast_notify == 'on'){
			add_action( 'send_order_to_trackship', array( $this, 'trigger_order_trackship' ), 10, 1 );
		}
	}

	/*add default settings to savesetting in setting-options*/
	public static function addDefaultSetting($defaults=array())
	{
		$defaults['smsalert_ast_general']['ast_notify']	= 'off';
		$defaults['smsalert_ast_message']['ast_notify']	= '';
		return $defaults;
	}

	/*add tabs to smsalert settings at backend*/
	public static function addTabs($tabs=array())
	{
		$ast_addon__param=array(
			'checkTemplateFor'	=> 'ast_addon',
			'templates'			=> self::getASTAddonTemplates(),
		);

		$tabs['woocommerce']['inner_nav']['ast_addon']['title']		  = 'AST Tracking Per Item';
		$tabs['woocommerce']['inner_nav']['ast_addon']['tab_section'] = 'astaddontemplates';
		$tabs['woocommerce']['inner_nav']['ast_addon']['tabContent']  = 'views/message-template.php';
		$tabs['woocommerce']['inner_nav']['ast_addon']['params']	  = $ast_addon__param;
		return $tabs;
	}

	public static function getASTAddonTemplates()
	{
		$current_val 		= smsalert_get_option( 'ast_notify', 'smsalert_ast_general', 'on');
		$checkboxNameId		= 'smsalert_ast_general[ast_notify]';
		$textareaNameId		= 'smsalert_ast_message[ast_notify]';

		$text_body 			= smsalert_get_option('ast_notify', 'smsalert_ast_message', sprintf(__('Shipped: %s has been dispatched via %s with tracking number %s. Track here %s',SmsAlertConstants::TEXT_DOMAIN), '[item_name_qty]', '[tracking_provider]', '[tracking_number]', '[tracking_link]'));

		$templates 			= array();

		$variables = WooCommerceCheckOutForm::getvariables();

		$templates['ast-addon']['title'] 			= 'When Tracking Information is added';
		$templates['ast-addon']['enabled'] 			= $current_val;
		$templates['ast-addon']['status'] 			= '';
		$templates['ast-addon']['text-body'] 		= $text_body;
		$templates['ast-addon']['checkboxNameId'] 	= $checkboxNameId;
		$templates['ast-addon']['textareaNameId'] 	= $textareaNameId;
		$templates['ast-addon']['moreoption'] 		= 1;
		$templates['ast-addon']['token'] 			= $variables;

		return $templates;
	}

	public function trigger_order_trackship( $order_id) {

		$order 			= new WC_Order( $order_id );
		$ast_message 	= smsalert_get_option('ast_notify', 'smsalert_ast_message','');

		$order_items 	= $order->get_items();
		$first_item 	= current($order_items);
		$post_id 		= $first_item['order_id'];
		$buyer_no 		= get_post_meta($post_id, '_billing_phone', true);

		do_action('sa_send_sms', $buyer_no, $this->parse_sms_body($ast_message, $order, $order_id));
	}

	public static function parse_sms_body($content, $order, $order_id)
	{
		$ast 			= new WC_Advanced_Shipment_Tracking_Actions;
		$tracking_items = $ast->get_tracking_items( $order_id,true );

		if(count($tracking_items)>0)
		{
			$t_info 	= end($tracking_items);
			if(array_key_exists('products_list',$t_info)){
				$item_with_qty 	= array();
				$item_name 		= array();
				foreach($t_info['products_list'] as $pdata){
					$item_with_qty[] = get_the_title ($pdata->product)." [".$pdata->qty."] ";
					$item_name[]	 = get_the_title ($pdata->product);
				}
				$item_with_qty 	= implode(",", $item_with_qty);
				$item_name 		= implode(",", $item_name);
			}
		}
		
		$find = array(
			'[item_name]',
			'[item_name_qty]',
		);

		$replace = array(
			$item_name,
			$item_with_qty,
		);

		$content = str_replace( $find, $replace, $content );

		$content = WooCommerceCheckOutForm::pharse_sms_body( $content, 'partial-shipped', $order, $order_note = '', $rma_id = '' );

		return $content;
	}
}
new ast_addon;
?>