<?php
if (! defined( 'ABSPATH' )) exit;

class smsalert_delivery_drivers_woocommerce
{
	public function __construct() {
		add_filter('sAlertDefaultSettings',  __CLASS__ .'::addDefaultSetting',1);
		add_filter('sa_wc_variables',  __CLASS__ .'::addTemplateVariable',1,2);

		$smsalert_driver_notify  = smsalert_get_option( 'driver_notify', 'smsalert_driver_general', 'on');

		if($smsalert_driver_notify == 'on'){
			add_action( 'woocommerce_order_status_changed', array( $this, 'trigger_onchange_order_status' ), 10, 3 );
		}
		add_action( 'sa_addTabs', array( $this, 'addTabs' ), 100 );
		add_action( 'ddwc_driver_dashboard_change_status_forms_bottom', array( $this, 'addCodeVerify' ), 100 );
		add_filter('sa_wc_order_sms_customer_before_send', __CLASS__ .'::modifySMSTextByOrderId',1,2);
		
		$this->verify_delivery_code();
	}
	
	public function addCodeVerify(){
		
		$order_id     		= filter_input( INPUT_GET, 'orderid' );
		$order        		= wc_get_order( $order_id );
		$order_data   		= $order->get_data();
		$order_status 		= $order_data['status'];
		
		$code_verify_enable =  metadata_exists( 'post', $order_id,  '_sa_deliverycode');
		
		$verify_code_status = get_post_meta($order_id, '_sa_deliverycode_status', true );
		if($order_status == 'out-for-delivery' && $verify_code_status == '0' && !empty($code_verify_enable)){
			echo '<script>
				jQuery(document).ready(function(){
					jQuery("input[name= ordercompleted]").parent("form").hide();
					jQuery("h4:last-of-type").hide();
				});
			</script>';
			
			$code_verify  = '<h4>' . esc_attr__( 'Code Verify', SmsAlertConstants::TEXT_DOMAIN ) . '</h4>';
			$code_verify .= '<form method="post" id="verify_code_form">';
			$code_verify .= '<input type="text" name="verify_code" value="" placeholder="' . esc_attr__( 'Enter delivery code', SmsAlertConstants::TEXT_DOMAIN ) . '" />';
			$code_verify .= '<input type="hidden" name="sa_verify_delivery_code" value="sa_verify_delivery_code"/>';
			$code_verify .= '<input type="hidden" name="order_id" value="'.$_GET['orderid'].'"/>';
			$code_verify .= '<input type="submit" name="code_submit" value="' . esc_attr__( 'Submit', SmsAlertConstants::TEXT_DOMAIN ) . '" class="button submit_code ddwc-change-status" />';
			$code_verify .= '</form>';
			
			echo $code_verify;
		}
	}
	
	public function verify_delivery_code(){
		if(!empty($_REQUEST['sa_verify_delivery_code']) && $_REQUEST['sa_verify_delivery_code']=='sa_verify_delivery_code'){
			
			$order_id 		= $_POST['order_id'];			
			$delivery_code 	= get_post_meta($order_id, $key = '_sa_deliverycode', $single = true );
			if($_POST['verify_code'] == $delivery_code){
				update_post_meta($order_id, "_sa_deliverycode_status", 1);
			}else{
				add_action( 'ddwc_driver_dashboard_change_status_forms_bottom', array( $this, 'code_verify_notice' ), 100 );
			}
		}
	}
	
	function code_verify_notice(){
		echo '<script>
			jQuery(document).ready(function(){
				var error = jQuery("<p>");
				error.text("Code Invalid!");
                error.addClass("ddwc-map-api-error-msg");
				jQuery(".ddwc-orders").prepend(error);
			});		
		</script>';
	}
	
	/*add tabs to smsalert settings at backend*/
	public static function addTabs($tabs=array())
	{
		$delivery_drivers_param=array(
			'checkTemplateFor'	=> 'delivery_drivers',
			'templates'			=> self::getDeliverDriversTemplates(),
		);
		
		$tabs['woocommerce']['inner_nav']['delivery']['title']		 = 'Delivery Drivers';
		$tabs['woocommerce']['inner_nav']['delivery']['tab_section'] = 'deliverydriverstemplates';
		$tabs['woocommerce']['inner_nav']['delivery']['tabContent']	 = 'views/message-template.php';
		$tabs['woocommerce']['inner_nav']['delivery']['params']	 	 = $delivery_drivers_param;
		return $tabs;
	}

	public static function getDeliverDriversTemplates()
	{
		$current_val 		= smsalert_get_option( 'driver_notify', 'smsalert_driver_general', 'on');
		$checkboxNameId		= 'smsalert_driver_general[driver_notify]';
		$textareaNameId		= 'smsalert_driver_message[driver_notify]';

		$text_body 			= smsalert_get_option('driver_notify', 'smsalert_driver_message', sprintf(__('Hi %s %s , The package for order %s Products %s %s has been ready to pickup from %s.',SmsAlertConstants::TEXT_DOMAIN), '[first_name]', '[last_name]', '[order_id]', '[item_name]', '[item_name_qty]', '[store_name]'));

		$templates 			= array();

		$variables = array(
			'[first_name]' 		=> 'First Name',
			'[last_name]' 		=> 'Last Name',
			'[order_id]' 		=> 'Order Id',
			'[item_name_qty]' 	=> 'Product Name with Quantity',
			'[item_name]' 		=> 'Product Name',
			'[store_name]' 		=> 'Store Name',
		);

		$templates['delivery-drivers']['title'] 			= 'When Order is assigned to driver';
		$templates['delivery-drivers']['enabled'] 			= $current_val;
		$templates['delivery-drivers']['status'] 			= 'driver-assigned';
		$templates['delivery-drivers']['text-body'] 		= $text_body;
		$templates['delivery-drivers']['checkboxNameId'] 	= $checkboxNameId;
		$templates['delivery-drivers']['textareaNameId'] 	= $textareaNameId;
		$templates['delivery-drivers']['token'] 			= $variables;
		
		return $templates;
	}

	public static function modifySMSTextByOrderId($params,$order_id)
	{
		if(empty($params['sms_body']))
			return $params;
		
		$order 				= new WC_Order( $order_id );
		$order_items 		= $order->get_items();
		$first_item 		= current($order_items);
		$post_id 			= $first_item['order_id'];
		$driver_id 			= get_post_meta( $post_id, 'ddwc_driver_id', true);
		$order_variables 	= get_user_meta($driver_id);
		$delivery_code 		= get_post_meta($post_id, $key = '_sa_deliverycode', $single = true );
		$find = array(
			'[delivery_first_name]',
			'[delivery_last_name]',
			'[delivery_boy_number]',
			'[delivery_code]',
		);

		$replace = array(
			'[first_name]',
			'[last_name]',
			'[billing_phone]',
			$delivery_code,
		);
		
		$content = str_replace( $find, $replace, $params['sms_body'] );
		foreach ($order_variables as &$value) {
			$value = $value[0];
		}
		unset($value);

		$order_variables = array_combine(
			array_map(function($key){ return '['.ltrim($key, '_').']'; }, array_keys($order_variables)),
			$order_variables
		);
        $content = str_replace( array_keys($order_variables), array_values($order_variables), $content );
		
		$params['sms_body'] = $content;
		return $params;
	}

	public static function addTemplateVariable($variables=array(), $status)
	{
		if($status == 'driver-assigned'){
			$variables = array_merge($variables,  array(
				'[delivery_first_name]' => 'Delivery Boy First Name',
				'[delivery_last_name]' 	=> 'Delivery Boy Last Name',
				'[delivery_boy_number]' => 'Delivery Boy Number',
				'[delivery_code]' 		=> 'Delivery Code',
			));
		}
		return $variables;
	}

	/*add default settings to savesetting in setting-options*/
	public static function addDefaultSetting($defaults=array())
	{
		$defaults['smsalert_driver_general']['driver_notify']	= 'off';
		$defaults['smsalert_driver_message']['driver_notify']	= '';
		return $defaults;
	}

	public  function trigger_onchange_order_status( $order_id, $old_status, $new_status ) {
		
		if($new_status == 'driver-assigned')
		{	
			$delivery_code = rand(1,9999);
			
			$order 			= new WC_Order( $order_id );
			$driver_message = smsalert_get_option('driver_notify', 'smsalert_driver_message','');

			$order_items 	= $order->get_items();
			$first_item 	= current($order_items);
			$post_id 		= $first_item['order_id'];
			$driver_id 		= get_post_meta( $post_id, 'ddwc_driver_id', true);
			$driver_no 		= get_the_author_meta('billing_phone', $driver_id);
			do_action('sa_send_sms', $driver_no, $this->parse_sms_body($order, $driver_message, $driver_id));
			$this->saveDeliveryCode($order_id,$delivery_code);
		}
	}
	
	public function saveDeliveryCode($order_id,$delivery_code){
		update_post_meta($order_id, "_sa_deliverycode", $delivery_code);
		update_post_meta($order_id, "_sa_deliverycode_status", 0);
	}

	public function parse_sms_body($order, $message, $driver_id){
		
		$order_items 	= $order->get_items();
		$item 			= current($order_items);
		$item_name 		= $item['name'];
		$order_id 		= $item['order_id'];
		$quantity 		= $item['quantity'];
		$first_name 	= get_the_author_meta('first_name', $driver_id);
		$last_name 		= get_the_author_meta('last_name', $driver_id);

		$find = array(
			'[first_name]',
			'[last_name]',
			'[item_name]',
			'[order_id]',
			'[item_name_qty]',
			'[store_name]',
		);

		$replace = array(
			$first_name,
			$last_name,
			$item_name,
			$order_id,
			$item_name.' '.$quantity,
			get_bloginfo(),
		);

		$message 	= str_replace($find, $replace, $message);
		return $message;
	}
}
new smsalert_delivery_drivers_woocommerce;
?>